function dtTonalsSave(Filename, tonals, varargin)
% dtTonalsSave(Filename, tonals, OptionalArgs)
% Save the list of tonals to the specified file.  The recommended 
% filename extension is '.ann' for annotation file format.  When
% Filename is set to [], a dialog will request a filename.
%
% The tonals is a linked list of tonals.  See the documentation for 
% details on how to create these.
%
% OptionalArgs:
%   'Comment', string - Comment to be stored with file
%   'Dialog', true|false - Display a dialog that prompts for a filename.
%      Defaults to false unless Filename is empty.
%   'Save', Mask - Specify parameters to be saved with each point.
%      Mask is a bitwise or of any of the TonalHeader fields:  TIME,
%      FREQ, SNR, PHASE.  If not specified, time and frequency only will
%      be saved.  
%    'Scores', Scores - Score for each tonal in tonals.
%    'Confidences', Confidences - Confdence value for each tonal (between
%         zero and one).
%    'Version', N - A version number associated with the caller's detector.
%         It is saved as a short integer, for more sophisticated version
%         schemes, it is suggested that the comment field be used.
%
% Example:
%  % Invoke code that generates a set of tonals and corresponding scores
%  [tonals, scores] = YourDetectionCode('somefile.wav');
%
%  % score times, frequencies, and scores
%  dtTonalSave('somefile.ann', tonals, 'Score', scores);
%
%  % as above, but save the snr of each time x freq node too!
%  dtTonalSave('somefile.ann', tonals, 'Score', scores, ...
%     'Save', bitor(TonalHeader.TIME, 
%                   bitor(TonalHeader.FREQ, TonalHeader.SNR)));


import tonals.*;

error(nargchk(2,Inf,nargin));

% Set up defaults
savemask = TonalHeader.DEFAULT;  % time x freq only
gui = isempty(Filename);  % graphical prompt?
scores = [];  % scores associated with each annotation
confidences = [];  % confidence associated with each annotation
comment = '';  % comment associated with set of annotations
version = 0;  % not specified

% Parse the optional arguments, possibly overriding defaults
vidx = 1;
while vidx < length(varargin)
    switch varargin{vidx}
        case 'Comment'
            comment = varargin{vidx+1}; vidx = vidx+2;
        case 'Save'
            savemask = varargin{vidx+1}; vidx = vidx+2;
        case 'Scores'
            scores = varargin{vidx+1}; vidx = vidx+2;
            if length(scores) ~= tonals.size()
                error('Score vector must be same size as tonals');
            end
        case 'Confidences'
            confidences = varargin{vidx+1}; vidx = vidx+2;
            if length(confidences) ~= tonals.size()
                error('Confidence vector must be same size as tonals');
            end
        case 'Dialog'
            gui = varargin{vidx+1}; vidx = vidx+2;
        case 'Version'
            version = varragin{vidx+1};
            if ~isnumeric(version)
                error('integer required for version');
            end
        otherwise
            error('Bad optional argument');
    end
end

if ~isempty(scores)
    savemask = bitor(savemask, TonalHeader.SCORE);
end
if ~isempty(confidences)
    savemask = bitor(savemask, TonalHeader.CONFIDENCE);
end

% Handle graphical prompt if needed
pattern = {'*.ann', 'annotation'};
if gui
    [SaveFile, SaveDir] = uiputfile(pattern, 'Save Tonal Annotations', Filename);
    if isnumeric(SaveFile)
        return  % cancel
    end
    Filename = fullfile(SaveDir, SaveFile);
end

% open up file
tstream = TonalBinaryOutputStream(Filename, version, comment, savemask);

% Examine scores and confidences to determine what function to use
% 0 - no scores or confidences
% 1 - scores
% 2 - confidences
% 3 - scores and confidences
writeCode = bitor(bitshift(double(~isempty(confidences)), 1), double(~isempty(scores)));

% iterate through tonals
it = tonals.iterator();
count = 0;
while it.hasNext()
    count = count + 1;
    t = it.next();
    % write each tonal t
    switch writeCode
        case 0
            tstream.write(t);
        case 1
            tstream.write_s(t, scores(count));
        case 2
            tstream.write_c(t, confidences(count));
        case 3
            tstream.write_cs(t, confidences(count), scores(count));
    end
end
tstream.close();
