function tonal_test()
% tonal_test()
% Test tonal load/save functionality

% ========================================================================
% Generate a series of time/frequency sweeps, along with phony SNR & phase
% for test purposes.  Saved as a structure array for now.
% ton(i).
%   time - time in s
%   freq - frequency in Hz
%   snr - signal to noise ratio
%   phase - frequency's phase
N=20;
ton = generate_tonals(N);

% generate phony scores and confidences
scores = randn(N, 1)*20 + 200;
confidences = rand(N, 1);


% We have a Java representation of tonals 
tonalset = convert_to_tonal_obj(ton);

% ========================================================================
% Test various save and load formats

import tonals.TonalBinaryInputStream;
import tonals.TonalBinaryOutputStream;
import tonals.TonalHeader;

fname = [tempname, '.ann'];  % temporary filename than ends in .ann

% Test simple save
fprintf('Simple test\n');
dtTonalsSave(fname, tonalset);  % default, saves time x freq only
[values, meta] = dtTonalsLoad(fname);  % Load back in
verify(tonalset, values, meta, [], []);

% Next tests save everything
mask = bitor(TonalHeader.TIME, ...
    bitor(TonalHeader.FREQ, ...
    bitor(TonalHeader.SNR, TonalHeader.PHASE)));

% Try it again.  This time we save SNR and phase for each point
% along with a comment and scores
fprintf('Saving Scores\n');
fname2 = [tempname, '.ann'];
dtTonalsSave(fname2, tonalset, 'Save', mask, ...
    'Comment', 'tonal set w/scores', 'Scores', scores);
[values2, meta2] = dtTonalsLoad(fname2);
verify(tonalset, values2, meta2, scores, []);
fprintf('Comment:  %s\n', meta2.comment);


fprintf('Saving Confidences\n');
fname3 = [tempname, '.ann'];
dtTonalsSave(fname3, tonalset, 'Save', mask, ...
    'Comment', 'tonal set w/confidences', 'Confidences', confidences);
[values3, meta3] = dtTonalsLoad(fname3);
verify(tonalset, values3, meta3, [], confidences);
fprintf('Comment:  %s\n', meta3.comment);

fprintf('Saving scores and confidences\n');
fname4 = [tempname, '.ann'];
dtTonalsSave(fname4, tonalset, 'Save', mask, ...
    'Comment', 'tonal set w/ scores & confidences', ...
    'Scores', scores, 'Confidences', confidences);
[values4, meta4] = dtTonalsLoad(fname4);
verify(tonalset, values4, meta4, scores, confidences);
fprintf('Comment:  %s\n', meta4.comment);


% Example of streaming output
% Save time & freq along with score
mask = bitor(TonalHeader.DEFAULT, TonalHeader.SCORE);
fname5 = [tempname, '.ann'];
detversion = 5;  % detector version
ostream = tonals.TonalBinaryOutputStream(fname5, detversion, ...
    'Detection example', mask);
for idx = 1:length(ton)
    ostream.write_s(ton(idx).time, ton(idx).freq, scores(idx));
end
ostream.close();  % all done
[values5, meta5] = dtTonalsLoad(fname5);
verify(tonalset, values5, meta5, scores, []);
fprintf('Comment:  %s\n', meta5.comment);

    
% Finally, let's mix up the order before we save them out
% On loading back in, the order should be the same
order = randperm(N);    % Generate random permuation
% Reorder the arrays according to the permuation
pton = ton(order);
pscores = scores(order);
pconfidences = confidences(order);

fname6 = [tempname, '.ann'];
ptonallist = convert_to_tonal_obj(pton);
dtTonalsSave(fname6, ptonallist, 'Save', mask, ...
    'Comment', 'Tonals not written in order', ...
    'Scores', pscores, 'Confidences', pconfidences);
[values6, meta6] = dtTonalsLoad(fname6);
verify(tonalset, values6, meta6, scores, confidences);
fprintf('Comment:  %s\n', meta6.comment);

    

function tonalset = convert_to_tonal_obj(ton)
% Given a structure array of tonals,
% convert to a Java linked list of tonal objects
import tonals.tonal;

tonalset = java.util.LinkedList();
for idx=1:length(ton)
    % Note that we could have also tested this wihtout storing snr and
    % phase:  tonal(ton(idx).time, ton(idx).freq)
    atonal = tonal(ton(idx).time, ton(idx).freq, ton(idx).snr, ton(idx).phase);
    tonalset.add(atonal);
end



function ton = generate_tonals(N)
% ton = generate_tonals(N)
% Generate N random frequency sweeps

Fs = 48000;
frameAdv_s = .010;
durations_s = round(max(randn(N,1)*.3+1, .5)/frameAdv_s)*frameAdv_s;
spacing_s = 3;  % new tonal every N s
for idx=1:N
    
    % Generate a random tonal
    range_t = 0:frameAdv_s:durations_s(idx);        % time
    ton(idx).time = (idx-1)*spacing_s + range_t;
    f0 = randn(1,1)*2000 + 10000;                   % frequency
    %f1 = randn(1,1)*2000 + 10000;
    t2 = ton(idx).time(end);  %round(length(ton(idx).time)*.5)
    % Pick a curvature model and use chirp to produce it.
    polynomial = rand(1, randi([2,4]))*1000;  % Random order & coeff
    polynomial(end) = f0; % Always start at f0
    ton(idx).freq = polyval(polynomial, range_t);
    
    % We could compute relalistic versions for these, but we don't care...
    ton(idx).snr = rand(size(ton(idx).freq));
    ton(idx).phase = rand(size(ton(idx).freq));
end


function verify(orig, dup, meta, scores, confidences)
% verify(orig, dup, meta, scores, confidences)
% Verify that orignal and duplicate tonals are identical.

% Verify size
N = orig.size();
if N ~= dup.size()
    error('Number of tonals does not correspond');
end

% Verify values
for idx=0:N-1
    o = orig.get(idx);
    d = dup.get(idx);
    
    % Extract and compare values
    methods = {'get_time', 'get_freq', 'get_snr', 'get_phase'};
    valuesexist = {'hasTime', 'hasFreq', 'hasSNR', 'hasPhase'};
    for midx = 1:length(methods)
        if meta.hdr.(valuesexist{midx})
            ovals = o.(methods{midx});
            dvals = d.(methods{midx});
            if ovals ~= dvals
                error('Mismatch tonal idx %d, %s', midx, methods{midx});
            end
        end
    end    
end

if ~isempty(scores)
    if scores ~= meta.scores;
        error('scores do not match');
    end
end

if ~isempty(confidences)
    if confidences ~= meta.confidences;
        error('confidences do not match');
    end
end


1;

